@@ -13,7 +13,7 @@ val env : scenv -> env
 
 val initial : env -> scenv
 
-val add_item     : theory_item -> scenv -> scenv
+val add_item     : ?override_locality:EcTypes.is_local option -> theory_item -> scenv -> scenv
 val add_decl_mod : EcIdent.t -> mty_mr -> scenv -> scenv
 
 val enter_section : EcSymbols.symbol option -> scenv -> scenv
