@@ -19,7 +19,7 @@ type 'a ovrenv = {
   ovre_prefix   : (symbol list) EcUtils.pair;
   ovre_glproof  : (ptactic_core option * evtags option) list;
   ovre_abstract : bool;
-  ovre_local    : EcTypes.is_local;
+  ovre_local    : EcTypes.is_local option;
   ovre_hooks    : 'a ovrhooks;
 }
 
@@ -33,8 +33,8 @@ and 'a ovrhooks = {
 
 (* -------------------------------------------------------------------- *)
 val replay : 'a ovrhooks
-  -> abstract:bool -> local:is_local -> incl:bool
+  -> abstract:bool -> override_locality:EcTypes.is_local option -> incl:bool
   -> clears:Sp.t -> renames:(renaming list)
   -> opath:path -> npath:path -> evclone
-  -> 'a -> symbol * theory_item list
+  -> 'a -> symbol * theory_item list * EcTypes.is_local
   ->  axclone list * 'a
